"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_optional_1 = __importDefault(require("typescript-optional"));
function cons(a, d) {
    return [a].concat(d);
}
exports.cons = cons;
Object.defineProperty(Array.prototype, "head", {
    get() {
        return typescript_optional_1.default.ofNullable(this[0]);
    }
});
Object.defineProperty(Array.prototype, "last", {
    get() {
        return typescript_optional_1.default.ofNullable(this[this.length - 1]);
    }
});
Object.defineProperty(Array.prototype, "tail", {
    get() {
        return this.slice(1);
    }
});
Object.defineProperty(Array.prototype, "init", {
    get() {
        return this.slice(0, -1);
    }
});
Object.defineProperty(Array.prototype, "uncons", {
    get() {
        if (0 < this.length) {
            return typescript_optional_1.default.ofNonNull([this[0], this.tail]);
        }
        return typescript_optional_1.default.empty();
    }
});
Object.defineProperty(Array.prototype, "null", {
    get() {
        return this.length === 0;
    }
});
Object.defineProperty(Array.prototype, "subsequences", {
    get() {
        function nonEmptySubsequences(xa) {
            return xa.uncons.matches({
                empty: () => [],
                present: ([x, xs]) => {
                    const f = (ys, r) => cons(ys, cons(cons(x, ys), r));
                    return cons([x], nonEmptySubsequences(xs).reduceRight((prev, curr) => f(curr, prev), []));
                }
            });
        }
        return cons([], nonEmptySubsequences(this));
    }
});
Object.defineProperty(Array.prototype, "permutations", {
    get() {
        function perms(ta, is) {
            return ta.uncons.matches({
                empty: () => [],
                present: ([t, ts]) => {
                    function interleave2(f, za, r) {
                        return za.uncons.matches({
                            empty: () => {
                                const result = [ts, r];
                                return result;
                            },
                            present: ([z, zs]) => {
                                const [ua, va] = interleave2(a => f(cons(z, a)), zs, r);
                                const result = [
                                    cons(z, ua),
                                    cons(f(cons(t, cons(z, ua))), va)
                                ];
                                return result;
                            }
                        });
                    }
                    function interleave(ya, r) {
                        return interleave2(a => a, ya, r)[1];
                    }
                    return is.permutations.reduceRight((prev, curr) => interleave(curr, prev), perms(ts, cons(t, is)));
                }
            });
        }
        return cons(this, perms(this, []));
    }
});
Object.defineProperty(Array.prototype, "and", {
    get() {
        return this.every((e) => Boolean(e));
    }
});
Object.defineProperty(Array.prototype, "or", {
    get() {
        return this.some((e) => Boolean(e));
    }
});
Array.prototype.fmap = function (mapper) {
    return this.map(e => mapper(e));
};
Array.prototype.intersperse = function (inter) {
    const result = [];
    this.forEach((element, index) => {
        result.push(element);
        if (index !== this.length - 1) {
            result.push(inter);
        }
    });
    return result;
};
Array.prototype.takeWhile = function (pred) {
    const takeIndex = this.findIndex(e => !pred(e));
    if (takeIndex === -1) {
        return [...this];
    }
    else {
        return this.slice(0, takeIndex);
    }
};
//# sourceMappingURL=index.js.map